#ifndef STRTOIP_H____
#define STRTOIP_H____

extern int strtoip4(unsigned char *, const char *);
extern int strtoip6(unsigned char *, const char *);
extern int strtoip(unsigned char *, const char *);

#endif
