#ifndef CONNECTIONINFO_H____
#define CONNECTIONINFO_H____

extern int connectioninfo_get(unsigned char *, unsigned char *, unsigned char *,
                              unsigned char *);
extern void connectioninfo_set(unsigned char *, unsigned char *,
                               unsigned char *, unsigned char *);

#endif
