#ifndef MAIN_H____
#define MAIN_H____

extern int main_tlswrapper(int, char **, int);
extern int main_tlswrapper_tcp(int, char **, int);
extern int main_tlswrapper_smtp(int, char **, int);
extern int main_tlswrapper_test(int, char **);

#endif
