#ifndef PIPE_H____
#define PIPE_H____

extern int pipe_write(int, const void *, long long);
extern int pipe_writeerrno(int);

extern int pipe_readall(int, void *, size_t);
extern int pipe_readmax(int, void *, size_t *);
extern void *pipe_readalloc(int, size_t *);

#endif
